// ePDFView - A lightweight PDF Viewer.
// Copyright (C) 2006 Emma's Software.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <config.h>
#include <string.h>
#include "epdfview.h"

using namespace ePDFView;

static const gint bytesPerPixel = 3;

///
/// @brief Constructs a new DocumentPage.
///
DocumentPage::DocumentPage ()
{
    m_Data = NULL;
    m_Height = 0;
    m_Width = 0;
}

///
/// @brief Deletes all dynamically allocated memory for DocumentPage.
///
DocumentPage::~DocumentPage ()
{
    delete[] m_Data;
}

///
/// @brief Gets the page's data.
///
/// Gets the actual pixels the page is formed of. The format of the image
/// is RGB, using 3 bytes per pixel and a total of getWidth() * getHeight()
/// pixels.
///
/// @return The page's pixels array.
///
guchar *
DocumentPage::getData ()
{
    g_assert ( NULL != m_Data && 
              "Tried to retrieve data without creating a new page.");

    return m_Data;
}

///
/// @brief Gets the page's height.
///
/// @return The height of the page's image.
///
gint
DocumentPage::getHeight ()
{
    return m_Height;
}

///
/// @brief Gets the row stride.
///
/// The row stride is the bytes between two consecutive rows of the page's 
/// image retrieved by getData(). Currently this value is 3 * getWidth(), 
/// because I use 3 bytes per pixel, but this can change, so better use this
/// function.
///
/// @return The number of bytes between two consecutive rows.
///
gint
DocumentPage::getRowStride ()
{
    return bytesPerPixel * getWidth ();
}

///
/// @brief Gets the page's width.
///
/// @return The width of the page's image.
///
gint
DocumentPage::getWidth ()
{
    return m_Width;
}

///
/// @brief Allocates the memory for a new page.
///
/// It also stores the @a width and @a height, to be retrieved by
/// calling getWidth() and getHeight() respectively.
///
/// @param width The width of the new page.
/// @param height The height of the new page.
///
/// @return true if the image could be created. False otherwise.
///
gboolean
DocumentPage::newPage (gint width, gint height)
{
    g_assert ( 1 <= width && "Tried to create a 0 width page.");
    g_assert ( 1 <= height && "Tried to create a 0 height page.");
    
    m_Width = width;
    m_Height = height;

    gint dataSize = width * height * bytesPerPixel;
    delete[] m_Data;
    m_Data = new guchar[width * height * bytesPerPixel];
    if ( NULL != m_Data )
    {
        memset (m_Data, 0, dataSize);
    }
    
    return NULL != m_Data;
}
