// ePDFView - A lightweight PDF Viewer.
// Copyright (C) 2006 Emma's Software.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <config.h>
#include <stdlib.h>
#include "epdfview.h"

using namespace ePDFView;

///
/// @brief Constructs a new main presenter with a default document.
///
MainPter::MainPter ()
{
    m_Document = new PDFDocument ();
    m_DocumentPage = NULL;
    m_View = NULL;
}

///
/// @brief Constructs a new main presenter with the give document.
///
/// @param document The document to view. It doesn't have to be loaded,
///                 as it can be used for testing purposes.
///                 This document will be freed by the presenter.
///
MainPter::MainPter (IDocument *document)
{
    g_assert (NULL != document && "Tried to set a NULL document");

    m_Document = document;
    m_DocumentPage = NULL;
    m_View = NULL;
}

///
/// @brief Deletes all dynamically allocated memory for the presenter.
///
MainPter::~MainPter ()
{
    delete m_Document;
    delete m_View;
    delete m_DocumentPage;
}

///
/// @brief Initialises the view with the initial application's state.
///
/// Normally that will make all action except "Open File" insensitive, will
/// set the window's title and tell the main view to show itself.
///
/// @param canShowPage Set to TRUE if the application should show the page 
///                    after setting the initial state or not. This is useful 
///                    when reloading the document, because after loading we 
///                    want to set the page, rotation and zoom again and then
///                    show the page again.
///
void
MainPter::setInitialState (gboolean canShowPage)
{
    g_assert ( NULL != m_Document && "The document is NULL.");
    
    IMainView &view = getView ();
    gboolean showSidebar = FALSE;
    if ( m_Document->isLoaded () )
    {
        m_Document->goToFirstPage ();
        if ( 0 == g_ascii_strcasecmp ("", m_Document->getTitle ()) )
        {
            view.setTitle (m_Document->getFileName ());
        }
        else
        {
            view.setTitle (m_Document->getTitle ());
        }
        view.sensitiveGoToPage (TRUE);
        view.sensitiveReload (TRUE);
        view.sensitiveRotateRight (TRUE);
        view.sensitiveRotateLeft (TRUE);
        view.sensitiveZoomIn (TRUE);
        view.sensitiveZoomOut (TRUE);
        view.sensitiveZoomFit (TRUE);
        view.sensitiveZoomWidth (TRUE);
        view.setTotalPages (m_Document->getNumPages ());

        // Check if we should see the outlines.
        showSidebar = (NULL != m_Document->getOutline () &&
                       0 < m_Document->getOutline ()->getNumChildren () &&
                       PageModeOutlines == m_Document->getPageMode ());

        if ( canShowPage )
        {
            showPage ();
        }
    }
    else
    {
        view.setTitle (_("PDF Viewer"));
        view.sensitiveGoToFirstPage (FALSE);
        view.sensitiveGoToLastPage (FALSE);
        view.sensitiveGoToNextPage (FALSE);
        view.sensitiveGoToPage (FALSE);
        view.sensitiveGoToPreviousPage (FALSE);
        view.sensitiveReload (FALSE);
        view.sensitiveRotateRight (FALSE);
        view.sensitiveRotateLeft (FALSE);
        view.sensitiveZoomIn (FALSE);
        view.sensitiveZoomOut (FALSE);
        view.sensitiveZoomFit (FALSE);
        view.sensitiveZoomWidth (FALSE);
        showSidebar = FALSE;
    }
   
    view.setOutline (m_Document->getOutline ());
    view.showSidebar (showSidebar);
    view.show (); 
}

///
/// @brief Gets the presenter's view.
///
/// @return The reference to the view that was set with setView().
///
IMainView &
MainPter::getView ()
{
    g_assert (NULL != m_View && "Tried to get the view but it's NULL.");

    return (*m_View);
}

///
/// @brief Sets the presenter's view.
///
/// @param view The view that will receive the user events for the presenter.
///             This view will be deleted by the presenter.
///
void
MainPter::setView (IMainView *view)
{
    g_assert (NULL != view && "Tried to set a NULL view.");

    m_View = view;
    // Now that the view is set, it's time to set the initial 
    // state.
    setInitialState (TRUE);
}

///
/// @brief The "Go To First Page" action was activated.
///
void
MainPter::goToFirstPageActivated (void)
{
    g_assert (NULL != m_Document && 
              "Tried to go to the first page of a NULL document.");
    
    m_Document->goToFirstPage ();
    showPage ();
}

///
/// @brief The "Go To Last Page" action was activated.
///
void
MainPter::goToLastPageActivated (void)
{
    g_assert (NULL != m_Document && 
              "Tried to go to the last page of a NULL document.");
    
    m_Document->goToLastPage ();
    showPage ();
}

///
/// @brief The "Go To Next Page" action was activated.
///
void
MainPter::goToNextPageActivated (void)
{
    g_assert (NULL != m_Document && 
              "Tried to go to the next page of a NULL document.");
    
    m_Document->goToNextPage ();
    showPage ();
}

///
/// @brief The "Go To Page" action was activated.
///
/// This action is activated when the user presses the Enter key on
/// the displayed current page number. The user can change the value,
/// and the page will change to that value.
///
void
MainPter::goToPageActivated (void)
{
    g_assert (NULL != m_Document && 
              "Tried to go to the a page of a NULL document.");
    
    // First try to get the page number from the view.
    const gchar *goToPageText = getView ().getGoToPageText ();
    if ( NULL != goToPageText )
    {
        // Try to read as much as possible.
        int pageNum = atoi (goToPageText);
        // If the page number is too high, the Document class will
        // go to the last page. The same if it's too low.
        m_Document->goToPage (pageNum);
        showPage ();
    }
}


///
/// @brief The "Go To Previous Page" action was activated.
///
void
MainPter::goToPreviousPageActivated (void)
{
    g_assert (NULL != m_Document && 
              "Tried to go to the previous page of a NULL document.");
    
    m_Document->goToPreviousPage ();
    showPage ();
}

///
/// @brief The Open File action was activated.
///
/// This means that the user wants to open a new file. The presenter
/// asks the view to show the Open File dialog and return a file name.
/// Then we'll try to open the file and if it's correct we'll show it, 
/// otherwise it will request the view to show an error dialog or a 
/// password dialog, if the document is encrypted.
///
void
MainPter::openFileActivated ()
{
    IMainView &view = getView ();
    gchar *fileName = view.openFileDialog ();
    openDocument (fileName, NULL, TRUE);
    g_free (fileName);
}

///
/// @brief The user activated an outline.
///
/// When the sidebar is shown and the user clicks on a outline, the view
/// should send to the presenter which outline has been activated, so the
/// presenter can change the page to the pointed by the outline.
///
/// @param outline The outline that has been activated.
void
MainPter::outlineActivated (DocumentOutline *outline)
{
    g_assert (NULL != outline && "The outline activated is NULL.");
    
    m_Document->goToPage (outline->getDestinationPage ());
    showPage ();
}

///
/// @brief The "Reload" was activated.
///
/// Reloading is like opening the same file but the current page, rotation
/// and zoom are maintained.
///
void
MainPter::reloadActivated ()
{    
    // Save the current document's state.
    gint currentPage = m_Document->getCurrentPageNum ();
    gint currentRotation = m_Document->getRotation ();
    gdouble currentZoom = m_Document->getZoom ();
    // Reopen the document.
    openDocument (m_Document->getFileName (), m_Document->getPassword (), 
                  FALSE); 
    // And restore the state
    m_Document->setZoom (currentZoom);
    m_Document->setRotation (currentRotation);
    m_Document->goToPage (currentPage);
    // Draw the document.
    showPage ();
}

///
/// @brief The "Rotate Left" was activated.
///
void
MainPter::rotateLeftActivated ()
{
    g_assert ( NULL != m_Document && "Tried to rotate a NULL document.");

    m_Document->rotateLeft ();
    showPage ();
}

///
/// @brief The "Rotate Right" was activated.
///
void
MainPter::rotateRightActivated ()
{
    g_assert ( NULL != m_Document && "Tried to rotate a NULL document.");

    m_Document->rotateRight ();
    showPage ();
}

///
/// @brief The "Zoom Fit Best" was activated.
///
void
MainPter::zoomFitActivated (void)
{
    g_assert ( NULL != m_Document && "Tried to zoom fit a NULL document.");

    gint width;
    gint height;
    getView ().getPageViewSize (&width, &height);
    m_Document->zoomToFit (width, height);
    showPage ();
}

///
/// @brief The "Zoom In" was activated.
///
void
MainPter::zoomInActivated (void)
{
    g_assert ( NULL != m_Document && "Tried to zoom in a NULL document.");

    m_Document->zoomIn ();
    showPage ();
}

///
/// @brief The "Zoom Out" was activated.
///
void
MainPter::zoomOutActivated (void)
{
    g_assert ( NULL != m_Document && "Tried to zoom out a NULL document.");

    m_Document->zoomOut ();
    showPage ();
}

///
/// @brief The "Zoom Fit Width" was activated.
///
void
MainPter::zoomWidthActivated (void)
{
    g_assert ( NULL != m_Document && "Tried to zoom width a NULL document.");

    gint width;
    gint height;
    getView ().getPageViewSize (&width, &height);
    m_Document->zoomToWidth (width);
    showPage ();
}

///
/// @brief Opens a document.
///
/// This function is called both by openFileActivated() and reloadActivated
/// to open a document.
///
/// @param fileName If be the name of the document to open. If it's
///                 NULL then that means the user didn't wanted to
///                 open a file, so this function won't do anything.
/// @param oldPassword This is only used when reloading. It is the last
///                    password that was used to open a file, and will be
///                    used to open the file the first try before asking
///                    to the user. For opening a new file, just set to NULL.
/// @param canShowPage Set to TRUE if the application should show the page 
///                    after the loading or not. This is useful when reloading
///                    the document, because after loading we want to set the
///                    page, rotation and zoom again and then show the page.
///
void
MainPter::openDocument (const gchar *fileName, const gchar *oldPassword,
                        gboolean canShowPage)
{
    // if fileName is NULL, then the user cancelled the operation.
    // I don't need to do anything in this case. I'm only interested when
    // the user tried to open a file.
    if ( NULL != fileName )
    {
        GError *error = NULL;        
        if ( m_Document->loadFile (fileName, oldPassword, &error) )
        {
            // Now that the document has been loaded, just reset the initial
            // state.
            setInitialState (canShowPage);
        }
        else
        {
            // We got an error, but also can be that the file is encrypted.
            if ( DocumentErrorEncrypted == error->code )
            {
                // Now we got an encrypted file.
                // Prompt for the password up to three time, until the password
                // is correct or the user cancels the password dialog.
                int passwordTries = 0;
                gchar *password = NULL;
                gboolean loaded = FALSE;
                do 
                {
                    g_free (password);
                    password = getView ().promptPasswordDialog ();
                    loaded = m_Document->loadFile (fileName, password, NULL);
                } while ( 3 > ++passwordTries && 
                          NULL != password && 
                          !loaded);

                // We are go, just check if we could load it or show the
                // error.
                if ( loaded )
                {
                    // Phew, finally loaded. Set the initial state.
                    setInitialState (canShowPage);
                }
                else if ( NULL != password )
                {
                    // We didn't get the correct password, but the user
                    // tried, because the password is not NULL.
                    getView ().showErrorMessage (_("Error Loading File"),
                            _("The password you have supplier is not a"
                              " valid password for this file."));
                    g_free (password);
                }
            }
            else
            {
                // It was an error. Show the error message.
                getView ().showErrorMessage (_("Error Loading File"), 
                                             error->message);                

            }
            g_free (error);
        }
    }
}


///
/// @brief Shows the current page.
///
/// This function is called when the presenter needs to show a document's
/// page. It only gets the rendered page from the document and then
/// pass the resultant DocumentPage to the view.
///
void
MainPter::showPage ()
{
    g_assert (NULL != m_Document && 
              "Tried to show a page from a NULL document.");
    g_assert (m_Document->isLoaded () && 
              "Tried to show a page from a not loaded document.");

    IMainView &view = getView ();
    // Set the text for the current page.
    gint currentPage = m_Document->getCurrentPageNum ();
    gchar *goToPageText = g_strdup_printf ("%d", currentPage);
    view.setGoToPageText (goToPageText);
    g_free (goToPageText);
    
    // Set the page navigation sensitivity.
    view.sensitiveGoToFirstPage (1 < currentPage );
    view.sensitiveGoToPreviousPage (1 < currentPage );
    view.sensitiveGoToLastPage (m_Document->getNumPages() > currentPage);
    view.sensitiveGoToNextPage (m_Document->getNumPages() > currentPage);
    // And the zoom sensitivity.
    view.sensitiveZoomIn (m_Document->canZoomIn ());
    view.sensitiveZoomOut (m_Document->canZoomOut ());
    
    DocumentPage *oldPage = m_DocumentPage;
    m_DocumentPage = m_Document->renderPage ();
    if ( NULL != m_DocumentPage )
    {
        getView ().showPage (m_DocumentPage);
    }
    delete oldPage;
}
