// ePDFView - A lightweight PDF Viewer.
// Copyright (C) 2006 Emma's Software.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <config.h>
#include <stdlib.h>
#include <string.h>
#include <gettext.h>
#include <gtk/gtk.h>
#include <epdfview.h>
#include "StockIcons.h"
#include "MainView.h"

using namespace ePDFView;

// Constants.
static gint CURRENT_PAGE_POS = 5;
static gint PAGE_VIEW_PADDING = 12;
static gint SCROLLBARS_SIZE = 20;

// Forward declarations.
static void main_window_about_box_cb (GtkWidget *, gpointer);
static void main_window_go_to_first_page_cb (GtkWidget *, gpointer);
static void main_window_go_to_last_page_cb (GtkWidget *, gpointer);
static void main_window_go_to_next_page_cb (GtkWidget *, gpointer);
static void main_window_go_to_page_cb (GtkWidget *, gpointer);
static void main_window_go_to_previous_page_cb (GtkWidget *, gpointer);
static void main_window_reload_cb (GtkWidget *, gpointer);
static void main_window_rotate_left_cb (GtkWidget *, gpointer);
static void main_window_rotate_right_cb (GtkWidget *, gpointer);
static void main_window_open_file_cb (GtkWidget *, gpointer);
static void main_window_outline_cb (GtkTreeView *, GtkTreePath *, 
                                    GtkTreeViewColumn *, gpointer);
static void main_window_quit_cb (GtkWidget *, gpointer);
static void main_window_show_sidebar_cb (GtkToggleAction *, gpointer);
static void main_window_show_statusbar_cb (GtkToggleAction *, gpointer);
static void main_window_show_toolbar_cb (GtkToggleAction *, gpointer);
static void main_window_zoom_fit_cb (GtkWidget *, gpointer);
static void main_window_zoom_in_cb (GtkWidget *, gpointer);
static void main_window_zoom_out_cb (GtkWidget *, gpointer);
static void main_window_zoom_width_cb (GtkWidget *, gpointer);

// The actions for menus and toolbars.
static const GtkActionEntry g_NormalEntries[] = 
{
    { "FileMenu", NULL, N_("_File"), NULL, NULL, NULL },
    { "ViewMenu", NULL, N_("_View"), NULL, NULL, NULL },
    { "GoMenu", NULL, N_("_Go"), NULL, NULL, NULL },
    { "HelpMenu", GTK_STOCK_HELP, N_("_Help"), NULL, NULL, NULL },

    { "OpenFile", GTK_STOCK_OPEN, N_("_Open"), "<control>O",       
      N_("Open a PDF document"),
      G_CALLBACK (main_window_open_file_cb) },

    { "ReloadFile", GTK_STOCK_REFRESH, N_("_Reload"), "<control>R",
      N_("Reload the current document"),
      G_CALLBACK (main_window_reload_cb) },
    
    { "Quit", GTK_STOCK_CLOSE, N_("_Close"), "<control>W", 
      N_("Close this window"), 
      G_CALLBACK (main_window_quit_cb) },
    
    { "ZoomIn", GTK_STOCK_ZOOM_IN, N_("Zoom _In"), "plus", 
      N_("Enlarge the document"), 
      G_CALLBACK (main_window_zoom_in_cb) },

    { "ZoomOut", GTK_STOCK_ZOOM_OUT, N_("Zoom _Out"), "minus", 
      N_("Shrink the document"), 
      G_CALLBACK (main_window_zoom_out_cb) },
    
    { "ZoomFit", GTK_STOCK_ZOOM_FIT, N_("Zoom to _Fit"), NULL,
      N_("Make the current document fill the window"), 
      G_CALLBACK (main_window_zoom_fit_cb) },

    { "ZoomWidth", EPDFVIEW_STOCK_ZOOM_WIDTH, N_("Zoom to _Width"), NULL,
      N_("Make the current document fill the window width"), 
      G_CALLBACK (main_window_zoom_width_cb) },

    { "RotateRight", EPDFVIEW_STOCK_ROTATE_RIGHT, N_("Rotate _Right"), NULL, 
      N_("Rotate the document 90 degrees clockwise"), 
      G_CALLBACK (main_window_rotate_right_cb) },

    { "RotateLeft", EPDFVIEW_STOCK_ROTATE_LEFT, N_("Rotate _Left"), NULL, 
      N_("Rotate the document 90 degrees counter-clockwise"), 
      G_CALLBACK (main_window_rotate_left_cb) },

    { "GoToFirstPage", GTK_STOCK_GOTO_FIRST, N_("_First Page"), "<control>Home",
      N_("Go to the first page"), 
      G_CALLBACK (main_window_go_to_first_page_cb) },

    { "GoToNextPage", GTK_STOCK_GO_FORWARD, N_("_Next Page"), "<control>Page_Down", 
      N_("Go to the next page"), 
      G_CALLBACK (main_window_go_to_next_page_cb) },

    { "GoToPreviousPage", GTK_STOCK_GO_BACK, N_("_Previous Page"), "<control>Page_Up", 
      N_("Go to the previous page"),
      G_CALLBACK (main_window_go_to_previous_page_cb) },

    { "GoToLastPage", GTK_STOCK_GOTO_LAST, N_("_Last Page"), "<control>End",
      N_("Go to the last page"),
      G_CALLBACK (main_window_go_to_last_page_cb) },

    { "About", GTK_STOCK_ABOUT, N_("_About"), NULL, 
      N_("Display application's credits"), 
      G_CALLBACK (main_window_about_box_cb) }
};

static GtkToggleActionEntry g_ToggleEntries[] = 
{
    { "ShowToolBar", NULL, N_("Show _Toolbar"), NULL,
      N_("Show or hide the toolbar"),
      G_CALLBACK (main_window_show_toolbar_cb), TRUE },
    { "ShowStatusBar", NULL, N_("Show _Statusbar"), NULL,
      N_("Show or hide the statusbar"),
      G_CALLBACK (main_window_show_statusbar_cb), TRUE },
    { "ShowSideBar", NULL, N_("Show O_utline"), NULL,
      N_("Show or hide the document's outline"),
      G_CALLBACK (main_window_show_sidebar_cb), FALSE }
};

////////////////////////////////////////////////////////////////
// Interface Methods.
////////////////////////////////////////////////////////////////

MainView::MainView (MainPter *pter):
    IMainView (pter)
{
    // Initialise the stock items.
    epdfview_stock_icons_init ();
    // Create the main window.
    m_MainWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    setMainWindowIcon ();
    // Connect already the destroy signal.
    g_signal_connect (G_OBJECT (m_MainWindow), "destroy",
                      G_CALLBACK (main_window_quit_cb), NULL);
    // Create the main vertical box.
    m_MainBox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (m_MainWindow), m_MainBox);
    gtk_widget_show (m_MainBox);
    // Generate menu and tool bars.
    createUIManager ();
    // Set the accelerator to the main window.
    GtkAccelGroup *accelGroup = gtk_ui_manager_get_accel_group (m_UIManager);
    gtk_window_add_accel_group (GTK_WINDOW (m_MainWindow), accelGroup);
    // Add the menu bar and tool bar.
    GtkWidget *menuBar = gtk_ui_manager_get_widget (m_UIManager, "/MenuBar");
    gtk_box_pack_start (GTK_BOX (m_MainBox), menuBar, FALSE, FALSE, 0);
    gtk_widget_show (menuBar);
    GtkWidget *toolBar = gtk_ui_manager_get_widget (m_UIManager, "/ToolBar");
    gtk_box_pack_start (GTK_BOX (m_MainBox), toolBar, FALSE, FALSE, 0);
    gtk_widget_show (toolBar);
    // Add the current page tool item.
    createCurrentPage ();
    gtk_toolbar_insert (GTK_TOOLBAR (toolBar), m_CurrentPageToolItem,
                        CURRENT_PAGE_POS);
    gtk_widget_show_all (GTK_WIDGET (m_CurrentPageToolItem));
    gtk_toolbar_insert (GTK_TOOLBAR (toolBar), gtk_separator_tool_item_new (),
                        CURRENT_PAGE_POS + 1);
    // Create the page view
    GtkWidget *pageViewPaned = createPageView ();
    gtk_box_pack_start (GTK_BOX (m_MainBox), pageViewPaned, TRUE, TRUE, 0);
    gtk_widget_show (pageViewPaned);

    // The status bar don't do anything, yet. But without the application
    // look weird.
    m_StatusBar = gtk_statusbar_new ();
    gtk_box_pack_start (GTK_BOX (m_MainBox), m_StatusBar, FALSE, FALSE, 0);
    gtk_widget_show (m_StatusBar);
}

MainView::~MainView ()
{
    g_object_unref (G_OBJECT (m_UIManager));
}


gchar *
MainView::openFileDialog ()
{
    GtkWidget *openDialog = gtk_file_chooser_dialog_new (_("Open PDF File"),
            GTK_WINDOW (m_MainWindow),
            GTK_FILE_CHOOSER_ACTION_OPEN,
            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
            GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
            NULL);
    if ( GTK_RESPONSE_ACCEPT == gtk_dialog_run (GTK_DIALOG (openDialog)) )
    {
        gchar *fileName = gtk_file_chooser_get_filename (
                                GTK_FILE_CHOOSER (openDialog));
        gtk_widget_destroy (openDialog);
        return fileName;
    }
    gtk_widget_destroy (openDialog);
    return NULL;
}

gchar *
MainView::promptPasswordDialog ()
{
    /* Create the password dialog. */
    GtkWidget *primaryLabel = gtk_label_new ("");
    gtk_label_set_markup (GTK_LABEL (primaryLabel),
            "<span weight=\"bold\" size=\"larger\">Encrypted Document</span>");
    gtk_label_set_selectable (GTK_LABEL (primaryLabel), TRUE);
    gtk_misc_set_alignment (GTK_MISC (primaryLabel), 0.0, 0.0);
    GtkWidget *secondaryLabel = gtk_label_new ("This document is encrypted "
                                               "and can't be accessed. "
                                               "Please enter the document's "
                                               "password:");
    gtk_label_set_line_wrap (GTK_LABEL (secondaryLabel), TRUE);
    gtk_label_set_selectable (GTK_LABEL (secondaryLabel), TRUE);
    gtk_misc_set_alignment (GTK_MISC (secondaryLabel), 0.0, 0.0);
    GtkWidget *passwordEntry = gtk_entry_new ();
    gtk_entry_set_visibility (GTK_ENTRY (passwordEntry), FALSE);
    gtk_entry_set_activates_default (GTK_ENTRY (passwordEntry), TRUE);
    GtkWidget *vbox = gtk_vbox_new (FALSE, 12);
    gtk_box_pack_start (GTK_BOX (vbox), primaryLabel, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), secondaryLabel, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), passwordEntry, FALSE, FALSE, 6);

    GtkWidget *image = 
        gtk_image_new_from_stock (GTK_STOCK_DIALOG_AUTHENTICATION,
                                  GTK_ICON_SIZE_DIALOG);
    gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
    GtkWidget *hbox = gtk_hbox_new (FALSE, 12);
    gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
    gtk_widget_show_all (hbox);

    
    GtkWidget *passwordDialog = gtk_dialog_new_with_buttons (
            _("Password"), GTK_WINDOW (m_MainWindow),
            GTK_DIALOG_DESTROY_WITH_PARENT,
            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
            GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
            NULL);
    gtk_dialog_set_default_response (GTK_DIALOG (passwordDialog), 
                                     GTK_RESPONSE_ACCEPT);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (passwordDialog)->vbox),
                        hbox, FALSE, FALSE, 0);
    
    gchar *password = NULL;
    if ( GTK_RESPONSE_ACCEPT == gtk_dialog_run (GTK_DIALOG (passwordDialog)) )
    {
        password = g_strdup(gtk_entry_get_text (GTK_ENTRY (passwordEntry)));
    }
    gtk_widget_destroy (passwordDialog);
    return password;
}

void 
MainView::sensitiveGoToFirstPage (gboolean sensitive)
{
    GtkAction *goToFirstPage = gtk_ui_manager_get_action (m_UIManager,
            "/MenuBar/GoMenu/GoToFirstPage");
    gtk_action_set_sensitive (goToFirstPage, sensitive);
}

void 
MainView::sensitiveGoToLastPage (gboolean sensitive)
{
    GtkAction *goToLastPage = 
        gtk_ui_manager_get_action (m_UIManager, "/MenuBar/GoMenu/GoToLastPage");
    gtk_action_set_sensitive (goToLastPage, sensitive);
}

void 
MainView::sensitiveGoToNextPage (gboolean sensitive)
{
    GtkAction *goToNextPage = 
        gtk_ui_manager_get_action (m_UIManager, "/MenuBar/GoMenu/GoToNextPage");
    gtk_action_set_sensitive (goToNextPage, sensitive);
}

void 
MainView::sensitiveGoToPage (gboolean sensitive)
{
    gtk_widget_set_sensitive (GTK_WIDGET (m_CurrentPageToolItem), sensitive);
}

void 
MainView::sensitiveGoToPreviousPage (gboolean sensitive)
{
    GtkAction *goToPreviousPage = gtk_ui_manager_get_action (m_UIManager, 
            "/MenuBar/GoMenu/GoToPreviousPage");
    gtk_action_set_sensitive (goToPreviousPage, sensitive);
}

void 
MainView::sensitiveReload (gboolean sensitive)
{
    GtkAction *reload = 
        gtk_ui_manager_get_action (m_UIManager, "/MenuBar/FileMenu/ReloadFile");
    gtk_action_set_sensitive (reload, sensitive);
}

void 
MainView::sensitiveRotateLeft (gboolean sensitive)
{
    GtkAction *rotateLeft = 
        gtk_ui_manager_get_action (m_UIManager, "/MenuBar/ViewMenu/RotateLeft");
    gtk_action_set_sensitive (rotateLeft, sensitive);
}

void 
MainView::sensitiveRotateRight (gboolean sensitive)
{
    GtkAction *rotateRight = gtk_ui_manager_get_action (m_UIManager, 
            "/MenuBar/ViewMenu/RotateRight");
    gtk_action_set_sensitive (rotateRight, sensitive);
}

void 
MainView::sensitiveZoomIn (gboolean sensitive)
{
    GtkAction *zoomIn = 
        gtk_ui_manager_get_action (m_UIManager, "/MenuBar/ViewMenu/ZoomIn");
    gtk_action_set_sensitive (zoomIn, sensitive);
}

void 
MainView::sensitiveZoomOut (gboolean sensitive)
{
    GtkAction *zoomOut = 
        gtk_ui_manager_get_action (m_UIManager, "/MenuBar/ViewMenu/ZoomOut");
    gtk_action_set_sensitive (zoomOut, sensitive);
}

void 
MainView::sensitiveZoomFit (gboolean sensitive)
{
    GtkAction *zoomFit = 
        gtk_ui_manager_get_action (m_UIManager, "/MenuBar/ViewMenu/ZoomFit");
    gtk_action_set_sensitive (zoomFit, sensitive);
}

void 
MainView::sensitiveZoomWidth (gboolean sensitive)
{
    GtkAction *zoomWidth = 
        gtk_ui_manager_get_action (m_UIManager, "/MenuBar/ViewMenu/ZoomWidth");
    gtk_action_set_sensitive (zoomWidth, sensitive);
}

void 
MainView::show (void)
{    
    gtk_widget_show (m_MainWindow);
    gtk_window_maximize (GTK_WINDOW (m_MainWindow));
}

void 
MainView::showErrorMessage (const gchar *title, const gchar *body)
{
    GtkWidget *errorDialog = gtk_message_dialog_new (
            GTK_WINDOW (m_MainWindow),
            GTK_DIALOG_DESTROY_WITH_PARENT,
            GTK_MESSAGE_ERROR,
            GTK_BUTTONS_OK,
            title);
    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG(errorDialog),
                                              body);
    gtk_dialog_run (GTK_DIALOG (errorDialog));
    gtk_widget_destroy (errorDialog);
}

void 
MainView::showPage (DocumentPage *page)
{
    gtk_image_set_from_pixbuf (GTK_IMAGE (m_PageView), NULL);
    GdkPixbuf *pixbuf = getPixbufFromPage (page); 
    gtk_image_set_from_pixbuf (GTK_IMAGE (m_PageView), pixbuf);
    gdk_pixbuf_unref (pixbuf);
    // Set the vertical scroll to the top of the page.
    GtkAdjustment *adjustment = gtk_scrolled_window_get_vadjustment (
                                    GTK_SCROLLED_WINDOW (m_PageViewScroll));
    gtk_adjustment_set_value (adjustment, adjustment->lower);
}

void
MainView::showSidebar (gboolean show)
{
    if ( show )
    {
        gtk_widget_show_all (m_Sidebar);
    }
    else
    {
        gtk_widget_hide (m_Sidebar);
    }
    GtkAction *showSideBar = 
        gtk_ui_manager_get_action (m_UIManager, 
                                   "/MenuBar/ViewMenu/ShowSideBar");
    gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (showSideBar), show);
}

void 
MainView::setTotalPages (gint pages)
{
    m_TotalPages = pages;
}

void 
MainView::setGoToPageText (const gchar *text)
{
    gchar *pageText = g_strdup_printf (_("%s of %d"), text, m_TotalPages);
    gtk_entry_set_text (GTK_ENTRY (m_CurrentPage), pageText);
    g_free (pageText);
}

const gchar *
MainView::getGoToPageText (void)
{
    return gtk_entry_get_text (GTK_ENTRY (m_CurrentPage));
}

void 
MainView::getPageViewSize (gint *width, gint *height)
{
    g_assert (NULL != width && "Tried to save the width to a NULL pointer.");
    g_assert (NULL != height && "Tried to save the height to a NULL pointer.");
   
    gint scrollSize = SCROLLBARS_SIZE + PAGE_VIEW_PADDING * 2;
    *width = m_PageViewScroll->allocation.width - scrollSize;
    *height = m_PageViewScroll->allocation.height - scrollSize;
}

void 
MainView::setTitle (const gchar *title)
{
    gtk_window_set_title (GTK_WINDOW (m_MainWindow), title);
}

void
MainView::setOutline (DocumentOutline *outline)
{
    gtk_tree_store_clear (m_Outline);
    setOutlineChildren (outline, NULL);
}

////////////////////////////////////////////////////////////////
// GTK+ Functions.
////////////////////////////////////////////////////////////////

///
/// @brief Creates the "Current Page" widget that will be displayed on toolbar.
///
/// @return The tool item to add to the toolbar.
///
void
MainView::createCurrentPage ()
{
    GtkWidget *hbox = gtk_hbox_new (FALSE, 3);
    
    GtkWidget *pageLabel= gtk_label_new (_("Page"));
    gtk_box_pack_start (GTK_BOX (hbox), pageLabel, FALSE, FALSE, 0);
    
    m_CurrentPage = gtk_entry_new ();
    gtk_entry_set_text (GTK_ENTRY (m_CurrentPage), _("0 of 0"));
    gtk_entry_set_alignment (GTK_ENTRY (m_CurrentPage), 1.0f);
    g_signal_connect (G_OBJECT (m_CurrentPage), "activate",
                      G_CALLBACK (main_window_go_to_page_cb), m_Pter);
    gtk_box_pack_start (GTK_BOX (hbox), m_CurrentPage, TRUE, TRUE, 0);
    
    m_CurrentPageToolItem = gtk_tool_item_new ();
    gtk_container_add (GTK_CONTAINER (m_CurrentPageToolItem), hbox);
}

///
/// @brief Creates the widget that will display the page.
///
/// Creates the image widget inside the scrolled window, but also creates
/// the sidebar.
///
/// Put both the sidebar and the image to a HPaned container.
///
/// @return The HPaned where the sidebar and the scrolled window are.
///
GtkWidget *
MainView::createPageView ()
{
    // Create the image view.
    m_PageViewScroll = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (m_PageViewScroll),
                                    GTK_POLICY_AUTOMATIC,
                                    GTK_POLICY_AUTOMATIC);
    m_PageView = gtk_image_new  ();
    gtk_misc_set_padding (GTK_MISC (m_PageView), 
                          PAGE_VIEW_PADDING, PAGE_VIEW_PADDING);
    gtk_scrolled_window_add_with_viewport (
            GTK_SCROLLED_WINDOW (m_PageViewScroll), m_PageView);

    // Create the side bar, with the outline tree.
    m_Outline = gtk_tree_store_new (3, G_TYPE_STRING, G_TYPE_INT, 
                                       G_TYPE_POINTER);
    GtkWidget *treeIndex = 
        gtk_tree_view_new_with_model (GTK_TREE_MODEL (m_Outline));
    gtk_widget_set_size_request (treeIndex, 200, -1);

    GtkTreeViewColumn *column = gtk_tree_view_column_new ();
    gtk_tree_view_column_set_expand (column, TRUE);
    gtk_tree_view_column_set_title (column, _("Index"));
    gtk_tree_view_append_column (GTK_TREE_VIEW (treeIndex), column);
   
    {
        GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
        GValue ellipsizeMode = { 0, };
        g_value_init (&ellipsizeMode, PANGO_TYPE_ELLIPSIZE_MODE);
        g_value_set_enum (&ellipsizeMode, PANGO_ELLIPSIZE_END);
        g_object_set_property (G_OBJECT (renderer),
                               "ellipsize", &ellipsizeMode);
        gtk_tree_view_column_pack_start (column, renderer, TRUE);
        gtk_tree_view_column_set_attributes (column, renderer, "text", 0, NULL);
    }
            
    {
        GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
        gtk_tree_view_column_pack_end (column, renderer, FALSE);
        gtk_tree_view_column_set_attributes (column, renderer, "text", 1, NULL);
    }
    // Set not selection for the tree index.
    GtkTreeSelection *selection = 
        gtk_tree_view_get_selection (GTK_TREE_VIEW (treeIndex));
    gtk_tree_selection_set_mode (selection, GTK_SELECTION_NONE);
    g_signal_connect (G_OBJECT (treeIndex), "row-activated",
                      G_CALLBACK (main_window_outline_cb), m_Pter);

    m_Sidebar = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (m_Sidebar),
                                    GTK_POLICY_AUTOMATIC,
                                    GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (m_Sidebar),
                                         GTK_SHADOW_IN);
    gtk_container_add (GTK_CONTAINER (m_Sidebar), treeIndex);


    GtkWidget *hPaned = gtk_hpaned_new ();
    gtk_paned_add1 (GTK_PANED (hPaned), m_Sidebar);
    gtk_paned_add2 (GTK_PANED (hPaned), m_PageViewScroll);
    gtk_widget_show_all (m_PageViewScroll);
    gtk_widget_show (hPaned);

    return hPaned;
}

///
/// @brief Creates and initialises the main window's UI manager.
///
void
MainView::createUIManager ()
{        
    GtkActionGroup *actionGroup = gtk_action_group_new ("ePDFView");
    // Set the translation domain for labels and tool tips.
    gtk_action_group_set_translation_domain (actionGroup, PACKAGE);
    // The data passed to all actions is the presenter
    gtk_action_group_add_actions (actionGroup, g_NormalEntries,
                                  G_N_ELEMENTS (g_NormalEntries), m_Pter);
    gtk_action_group_add_toggle_actions (actionGroup, g_ToggleEntries,
                                         G_N_ELEMENTS (g_ToggleEntries),
                                         this);
    m_UIManager = gtk_ui_manager_new ();
    gtk_ui_manager_insert_action_group (m_UIManager, actionGroup, 0);
    
    GError *error = NULL;
    if ( !gtk_ui_manager_add_ui_from_file (m_UIManager, 
                                           DATADIR"/ui/epdfview-ui.xml", 
                                           &error) )
    {
        g_critical (_("Error building UI manager: %s\n"), error->message);
        g_error_free (error);
        exit (EXIT_FAILURE);
    }

    // Set the previous and next pages tool items important, so they will
    // be shown when the toolbar's style is GTK_TOOLBAR_BOTH_HORIZ.
    GtkWidget *prevPage = gtk_ui_manager_get_widget (m_UIManager,
                                  "/ToolBar/GoToPreviousPage");
    gtk_tool_item_set_is_important (GTK_TOOL_ITEM (prevPage), TRUE);
    GtkWidget *nextPage = gtk_ui_manager_get_widget (m_UIManager, 
                                  "/ToolBar/GoToNextPage");
    gtk_tool_item_set_is_important (GTK_TOOL_ITEM (nextPage), TRUE);
}

///
/// @brief Creates a new GdkPixbuf from a given DocumentPage.
///
/// @param page The DocumentPage to transform to a GdkPixbuf.
/// @return The resultant GdkPixbuf.
///
GdkPixbuf *
MainView::getPixbufFromPage (DocumentPage *page)
{
    GdkPixbuf *pixbuf = gdk_pixbuf_new_from_data (page->getData (),
            GDK_COLORSPACE_RGB, FALSE, 8, page->getWidth (), page->getHeight (),
            page->getRowStride (), NULL, NULL);

    return pixbuf;
}

///
/// @brief Loads and sets the application's icon.
///
void
MainView::setMainWindowIcon ()
{
    const gchar *iconFiles[] =
    {
        "icon_epdfview-48.png",
        "icon_epdfview-32.png",
        "icon_epdfview-24.png",
    };
    
    GList *iconList = NULL;
    int iconFilesNum = G_N_ELEMENTS (iconFiles);
    for ( int iconIndex = 0 ; iconIndex < iconFilesNum ; iconIndex++ )
    {
        gchar *filename = g_strconcat (DATADIR, "/pixmaps/", 
                                       iconFiles[iconIndex], NULL);
        GError *error = NULL;
        GdkPixbuf *iconPixbuf = gdk_pixbuf_new_from_file (filename, &error);
        if ( NULL != iconPixbuf )
        {
            iconList = g_list_prepend (iconList, iconPixbuf);
        }
        else
        {
            g_warning ("Error loading icon: %s\n", error->message);
            g_error_free (error);
        }
        g_free (filename);
    }
    gtk_window_set_default_icon_list (iconList);
    g_list_foreach (iconList, (GFunc)g_object_unref, NULL);
    g_list_free (iconList);
}


void
MainView::setOutlineChildren (DocumentOutline *outline, GtkTreeIter *rootIter)
{
    if ( NULL != outline )
    {        
        DocumentOutline *child = outline->getFirstChild ();
        while (NULL != child)
        {
            GtkTreeIter childIter;            
            gtk_tree_store_append (m_Outline, &childIter, rootIter);
            gtk_tree_store_set (m_Outline, &childIter,
                                0, child->getTitle (),
                                1, child->getDestinationPage (),
                                2, child,
                                -1);
            setOutlineChildren (child, &childIter);
            child = outline->getNextChild ();
        }
    }
}

///
/// @brief Shows or hides the status bar.
///
/// @param show Set to TRUE to show the status bar. FALSE otherwise.
///
void
MainView::showStatusbar (gboolean show)
{
    if ( show ) 
    {
        gtk_widget_show (m_StatusBar);
    }
    else
    {
        gtk_widget_hide (m_StatusBar);
    }
}

///
/// @brief Shows or hides the tool bar.
///
/// @param show Set to TRUE to show the tool bar. FALSE otherwise.
///
void
MainView::showToolbar (gboolean show)
{
    GtkWidget *toolBar = gtk_ui_manager_get_widget (m_UIManager, "/ToolBar");
    if ( show ) 
    {
        gtk_widget_show (toolBar);
    }
    else
    {
        gtk_widget_hide (toolBar);
    }
}

////////////////////////////////////////////////////////////////
// GTK+ Callbacks.
////////////////////////////////////////////////////////////////

///
/// @brief The user tries to open the dialog box.
///
void
main_window_about_box_cb (GtkWidget *widget, gpointer data)
{
    const gchar *authors[] = {
        "Jordi Fita <jordi@emma-soft.com>",
        NULL
    };

    const gchar *comments = _("A lightweight PDF viewer");

    const gchar *license[] = {
        N_("ePDFView is free software; you can redistribute it and/or modify\n"
           "it under the terms of the GNU General Public License as published "
           "by\nthe Free Software Foundation; either version 2 of the "
           "License, or\n(at your option) any later version.\n"),
        N_("ePDFView is distributes in the hope that it will be useful,\n"
           "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
           "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
           "GNU General Public License for more details.\n"),
        N_("You should have received a copy of the GNU General Public License\n"
           "along with ePDFView; if not, write to the Free Software Foundation,"
           "Inc.,\n59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n")
    };
    gchar *licenseTranslated = g_strconcat (_(license[0]), "\n",
                                            _(license[1]), "\n",
                                            _(license[2]), "\n", NULL);    
    gtk_show_about_dialog (NULL,
            "name", _("ePDFView"),
            "version", VERSION,
            "copyright", "\xc2\xa9 2006 Emma's Software",
            "license", licenseTranslated,
            "website", "http://www.emma-soft.com/projects/epdfview/",
            "authors", authors,
            "comments", comments,
            NULL);

    g_free (licenseTranslated);
}

///
/// @brief The user tries to go to the first page.
///
void
main_window_go_to_first_page_cb (GtkWidget *widget, gpointer data)
{
    g_assert ( NULL != data && "The data parameter is NULL.");

    MainPter *pter = (MainPter *)data;
    pter->goToFirstPageActivated ();
}

///
/// @brief The user tries to go the last page.
///
void
main_window_go_to_last_page_cb (GtkWidget *widget, gpointer data)
{
    g_assert ( NULL != data && "The data parameter is NULL.");

    MainPter *pter = (MainPter *)data;
    pter->goToLastPageActivated ();
}

///
/// @brief The user tries to go to the next page.
///
void
main_window_go_to_next_page_cb (GtkWidget *widget, gpointer data)
{
    g_assert ( NULL != data && "The data parameter is NULL.");

    MainPter *pter = (MainPter *)data;
    pter->goToNextPageActivated ();
}

///
/// @brief The user tries to go to a given page on the toolbar.
///
void
main_window_go_to_page_cb (GtkWidget *widget, gpointer data)
{
    g_assert ( NULL != data && "The data parameter is NULL.");

    MainPter *pter = (MainPter *)data;
    pter->goToPageActivated ();
}

///
/// @brief The user tries to go to the previous page.
///
void
main_window_go_to_previous_page_cb (GtkWidget *widget, gpointer data)
{
    g_assert ( NULL != data && "The data parameter is NULL.");

    MainPter *pter = (MainPter *)data;
    pter->goToPreviousPageActivated ();
}

///
/// @brief The user tried to reload the current document.
///
void
main_window_reload_cb (GtkWidget *widget, gpointer data)
{
    g_assert ( NULL != data && "The data parameter is NULL.");

    MainPter *pter = (MainPter *)data;
    pter->reloadActivated ();
}

///
/// @brief The user tried to rotate the document counter-clockwise.
///
void
main_window_rotate_left_cb (GtkWidget *widget, gpointer data)
{
    g_assert ( NULL != data && "The data parameter is NULL.");

    MainPter *pter = (MainPter *)data;
    pter->rotateLeftActivated ();
}

///
/// @brief The user tried to rotate the document clockwise.
///
void
main_window_rotate_right_cb (GtkWidget *widget, gpointer data)
{
    g_assert ( NULL != data && "The data parameter is NULL.");

    MainPter *pter = (MainPter *)data;
    pter->rotateRightActivated ();
}

///
/// @brief The user tries to open a file.
///
void
main_window_open_file_cb (GtkWidget *widget, gpointer data)
{
    g_assert ( NULL != data && "The data parameter is NULL.");

    MainPter *pter = (MainPter *)data;
    pter->openFileActivated ();
}

///
/// @brief The user selected an outline index item.
///
void
main_window_outline_cb (GtkTreeView *treeIndex, GtkTreePath *treePath,
                        GtkTreeViewColumn *column, gpointer data)
{
    g_assert ( NULL != data && "The data parameter is NULL.");

    GtkTreeModel *indexModel = gtk_tree_view_get_model (treeIndex);
    GtkTreeIter iter;
    if ( gtk_tree_model_get_iter (indexModel, &iter, treePath) )
    {
        gpointer outlineData;
        gtk_tree_model_get (indexModel, &iter, 2, &outlineData, -1);
        DocumentOutline *outline = (DocumentOutline *)outlineData;
        MainPter *pter = (MainPter *)data;
        pter->outlineActivated (outline);
    }    
}

///
/// @brief Called when the window is closed or Quit is activated.
///
void
main_window_quit_cb (GtkWidget *widget, gpointer data)
{
    gtk_main_quit ();
}

///
/// @brief Called when the user clicks on the "Show Outline" action.
///
void
main_window_show_sidebar_cb (GtkToggleAction *action, gpointer data)
{
    g_assert ( NULL != data && "The data parameter is NULL.");

    MainView *view = (MainView *)data;
    view->showSidebar (gtk_toggle_action_get_active (action));
}

///
/// @brief Called when the user clicks on the "Show Statusbar" action.
///
void
main_window_show_statusbar_cb (GtkToggleAction *action, gpointer data)
{
    g_assert ( NULL != data && "The data parameter is NULL.");

    MainView *view = (MainView *)data;
    view->showStatusbar (gtk_toggle_action_get_active (action));
}

///
/// @brief Called when the user clicks on the "Show Toolbar" action.
///
void
main_window_show_toolbar_cb (GtkToggleAction *action, gpointer data)
{
    g_assert ( NULL != data && "The data parameter is NULL.");

    MainView *view = (MainView *)data;
    view->showToolbar (gtk_toggle_action_get_active (action));
}

///
/// @brief The user tries to fit the document into the window.
///
void
main_window_zoom_fit_cb (GtkWidget *widget, gpointer data)
{
    g_assert ( NULL != data && "The data parameter is NULL.");

    MainPter *pter = (MainPter *)data;
    pter->zoomFitActivated ();
}

///
/// @brief The user tries to expand the document.
///
void
main_window_zoom_in_cb (GtkWidget *widget, gpointer data)
{
    g_assert ( NULL != data && "The data parameter is NULL.");

    MainPter *pter = (MainPter *)data;
    pter->zoomInActivated ();
}

///
/// @brief The user tries to shrink the document.
///
void
main_window_zoom_out_cb (GtkWidget *widget, gpointer data)
{
    g_assert ( NULL != data && "The data parameter is NULL.");

    MainPter *pter = (MainPter *)data;
    pter->zoomOutActivated ();
}

///
/// @brief The user tries to fit the document in the window's width.
///
void
main_window_zoom_width_cb (GtkWidget *widget, gpointer data)
{
    g_assert ( NULL != data && "The data parameter is NULL.");

    MainPter *pter = (MainPter *)data;
    pter->zoomWidthActivated ();
}
