// ePDFView - Main Presenter Test Fixture.
// Copyright (C) 2006 Emma's Software.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <epdfview.h>
#include "DumbDocument.h"
#include "DumbMainView.h"
#include "MainPterTest.h"

using namespace ePDFView;

// Register the test suite into the `registry'.
CPPUNIT_TEST_SUITE_REGISTRATION (MainPterTest);

///
/// @brief Sets up the environment for each test.
///
void
MainPterTest::setUp ()
{
    m_Document = new DumbDocument ();
    m_MainPter = new MainPter (m_Document);
    m_View = new DumbMainView (m_MainPter);
    m_MainPter->setView (m_View);
}

///
/// @brief Cleans up after each test.
///
void
MainPterTest::tearDown ()
{
    // We only need to delete the presenter, as all other
    // classes are deleted by it.
    delete m_MainPter;
}


///
/// @brief Tests the initial status of the main presenter.
///
/// Initially the presenter hasn't loaded a document, yet, so
/// the main window's title is 'PDF Viewer', the page selection 
/// and zoom actions are insensitive and the documentView has
/// no image yet. Also the side bar won't be displayed, yet.
///
void
MainPterTest::initialStatus ()
{
    CPPUNIT_ASSERT ( m_View->isShown () );
    CPPUNIT_ASSERT_EQUAL (0, 
            g_ascii_strcasecmp ("PDF Viewer", m_View->getTitle ()));
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveReload ());
    CPPUNIT_ASSERT (!m_View->isSensitiveRotateLeft ());
    CPPUNIT_ASSERT (!m_View->isSensitiveRotateRight ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomIn ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomOut ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomFit ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomWidth ());
    CPPUNIT_ASSERT (!m_View->isShownSidebar ());
    CPPUNIT_ASSERT (!m_View->hasImagePageView ());
}

///
/// @brief Test loading a document.
///
/// Loading a document will set the title to the document's title or the
/// document's file name, if it has no title. Also will sensitive the
/// go to next page, go to last page and all zoom actions. Also will
/// show the document's first page. Since this document don't have
/// outline, the sidebar won't be displayed.
///
void
MainPterTest::loadDocument ()
{
    m_View->setOpenFileName ("/tmp/test.pdf");
    m_MainPter->openFileActivated ();
    CPPUNIT_ASSERT_EQUAL (0, 
            g_ascii_strcasecmp ("/tmp/test.pdf", m_View->getTitle ()));
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveReload ());
    CPPUNIT_ASSERT (m_View->isSensitiveRotateLeft ());
    CPPUNIT_ASSERT (m_View->isSensitiveRotateRight ());
    CPPUNIT_ASSERT (m_View->isSensitiveZoomIn ());
    CPPUNIT_ASSERT (m_View->isSensitiveZoomOut ());
    CPPUNIT_ASSERT (m_View->isSensitiveZoomFit ());
    CPPUNIT_ASSERT (m_View->isSensitiveZoomWidth ());
    CPPUNIT_ASSERT (!m_View->isShownSidebar ());
    CPPUNIT_ASSERT (m_View->hasImagePageView ());

    // Now try a document with a title.
    m_Document->setTitle (g_strdup ("Test PDF"));
    m_MainPter->openFileActivated ();
    CPPUNIT_ASSERT_EQUAL (0, 
            g_ascii_strcasecmp ("Test PDF", m_View->getTitle ()));
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveReload ());
    CPPUNIT_ASSERT (m_View->isSensitiveRotateLeft ());
    CPPUNIT_ASSERT (m_View->isSensitiveRotateRight ());
    CPPUNIT_ASSERT (m_View->isSensitiveZoomIn ());
    CPPUNIT_ASSERT (m_View->isSensitiveZoomOut ());
    CPPUNIT_ASSERT (m_View->isSensitiveZoomFit ());
    CPPUNIT_ASSERT (m_View->isSensitiveZoomWidth ());
    CPPUNIT_ASSERT (!m_View->isShownSidebar ());
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
}

///
/// @brief Tests a cancelled open file.
///
/// When the user cancels the open dialog, then nothing should change.
///
void
MainPterTest::loadCancelled ()
{
    // returning a NULL file name is what will happen when cancel.
    m_View->setOpenFileName (NULL);
    m_MainPter->openFileActivated ();
    CPPUNIT_ASSERT_EQUAL (0, 
            g_ascii_strcasecmp ("PDF Viewer", m_View->getTitle ()));
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveReload ());
    CPPUNIT_ASSERT (!m_View->isSensitiveRotateLeft ());
    CPPUNIT_ASSERT (!m_View->isSensitiveRotateRight ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomIn ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomOut ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomFit ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomWidth ());
    CPPUNIT_ASSERT (!m_View->hasImagePageView ());
    CPPUNIT_ASSERT (!m_View->shownError ());
}

///
/// @brief Test a failed load.
///
/// A failed load (anything but a DocumentErrorNone and DocumentErrorEncrypted)
/// is very similar to the load cancelled, but an error message is shown 
/// to the user.
///
void
MainPterTest::loadFailed ()
{    
    m_View->setOpenFileName ("/tmp/test.pdf");
    m_Document->setOpenError (DocumentErrorDamaged);
    m_MainPter->openFileActivated ();
    CPPUNIT_ASSERT_EQUAL (0, 
            g_ascii_strcasecmp ("PDF Viewer", m_View->getTitle ()));
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveReload ());
    CPPUNIT_ASSERT (!m_View->isSensitiveRotateLeft ());
    CPPUNIT_ASSERT (!m_View->isSensitiveRotateRight ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomIn ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomOut ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomFit ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomWidth ());
    CPPUNIT_ASSERT (!m_View->hasImagePageView ());
    CPPUNIT_ASSERT (m_View->shownError ());
}

///
/// @brief Test a cancelled password.
///
/// A cancelled password is more or less the same as cancelling the loading
/// of a file. It's when the user tried to open an encrypted file, but when 
/// the password is prompted, then it cancels :-)
/// It should happen the same as when cancelling the open file dialog.
///
void
MainPterTest::cancelledPassword ()
{
    m_View->setOpenFileName ("/tmp/test.pdf");
    m_View->setPassword (NULL);
    m_Document->setOpenError (DocumentErrorEncrypted);
    m_MainPter->openFileActivated ();
    CPPUNIT_ASSERT_EQUAL (0, 
            g_ascii_strcasecmp ("PDF Viewer", m_View->getTitle ()));
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveReload ());
    CPPUNIT_ASSERT (!m_View->isSensitiveRotateLeft ());
    CPPUNIT_ASSERT (!m_View->isSensitiveRotateRight ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomIn ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomOut ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomFit ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomWidth ());
    CPPUNIT_ASSERT (!m_View->hasImagePageView ());
    CPPUNIT_ASSERT (!m_View->shownError ());
}

///
/// @brief Test a bad password.
///
/// This test is very similar to the previous, but this time the user
/// enters a bad password, trying to guess which is the correct password.
/// The presenter gives up after three times and shows an error message.
///
void
MainPterTest::badPassword ()
{
    m_View->setOpenFileName ("/tmp/test.pdf");
    m_View->setPassword ("badpassword");
    m_Document->setTestPassword ("goodpassword");
    m_Document->setOpenError (DocumentErrorEncrypted);
    m_MainPter->openFileActivated ();
    CPPUNIT_ASSERT_EQUAL (0, 
            g_ascii_strcasecmp ("PDF Viewer", m_View->getTitle ()));
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveReload ());
    CPPUNIT_ASSERT (!m_View->isSensitiveRotateLeft ());
    CPPUNIT_ASSERT (!m_View->isSensitiveRotateRight ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomIn ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomOut ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomFit ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomWidth ());
    CPPUNIT_ASSERT (!m_View->hasImagePageView ());
    CPPUNIT_ASSERT (m_View->shownError ());
    CPPUNIT_ASSERT_EQUAL (3, m_View->countTimesShownPasswordPrompt ());
}

///
/// @brief Test a good password.
///
/// This test just check that giving the correct password opens the
/// file correctly.
///
void
MainPterTest::goodPassword ()
{
    m_View->setOpenFileName ("/tmp/test.pdf");
    m_View->setPassword ("goodpassword");
    m_Document->setTestPassword ("goodpassword");
    m_Document->setOpenError (DocumentErrorEncrypted);
    m_MainPter->openFileActivated ();
    CPPUNIT_ASSERT_EQUAL (0, 
            g_ascii_strcasecmp ("/tmp/test.pdf", m_View->getTitle ()));
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveReload ());
    CPPUNIT_ASSERT (m_View->isSensitiveRotateLeft ());
    CPPUNIT_ASSERT (m_View->isSensitiveRotateRight ());
    CPPUNIT_ASSERT (m_View->isSensitiveZoomIn ());
    CPPUNIT_ASSERT (m_View->isSensitiveZoomOut ());
    CPPUNIT_ASSERT (m_View->isSensitiveZoomFit ());
    CPPUNIT_ASSERT (m_View->isSensitiveZoomWidth ());
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    CPPUNIT_ASSERT (!m_View->shownError ());
    CPPUNIT_ASSERT_EQUAL (1, m_View->countTimesShownPasswordPrompt ());
}


///
/// @brief Test page navigation.
///
/// I'll load a 4-pages document and will try to go to the first, last, next, 
/// previous and also on a page using the entry on the toolbar.
///
void
MainPterTest::pageNavigation ()
{
    m_View->setOpenFileName ("/tmp/test.pdf");
    m_Document->setNumPages (4);
    m_MainPter->openFileActivated ();
    // Check that sets the correct number of pages and the current page.
    CPPUNIT_ASSERT_EQUAL (4, m_View->getTotalPages ());
    CPPUNIT_ASSERT_EQUAL (1, m_View->getCurrentPage ());
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    // This is to check that after calling this function, it returns false
    // because no page can be shown between the two... (I didn't called
    // the presenter or the view, yet)
    CPPUNIT_ASSERT (!m_View->hasImagePageView ());
    // Going to the next page should make all actions sensitive.
    m_MainPter->goToNextPageActivated ();
    CPPUNIT_ASSERT_EQUAL (2, m_View->getCurrentPage ()); 
    CPPUNIT_ASSERT (m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    // Going to the last will make some insensitive.
    m_MainPter->goToLastPageActivated ();
    CPPUNIT_ASSERT_EQUAL (4, m_View->getCurrentPage ()); 
    CPPUNIT_ASSERT (m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    // Again to the last won't harm (just in case...) nor going next.
    m_MainPter->goToLastPageActivated ();
    m_MainPter->goToNextPageActivated ();
    CPPUNIT_ASSERT_EQUAL (4, m_View->getCurrentPage ()); 
    CPPUNIT_ASSERT (m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    // But going to the previous will change something.
    m_MainPter->goToPreviousPageActivated ();
    CPPUNIT_ASSERT_EQUAL (3, m_View->getCurrentPage ()); 
    CPPUNIT_ASSERT (m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    CPPUNIT_ASSERT (!m_View->hasImagePageView ());
    // Now, to the first again.
    m_MainPter->goToFirstPageActivated ();
    CPPUNIT_ASSERT_EQUAL (1, m_View->getCurrentPage ()); 
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    // Check sane behaviour.
    m_MainPter->goToFirstPageActivated ();
    m_MainPter->goToPreviousPageActivated ();
    CPPUNIT_ASSERT_EQUAL (1, m_View->getCurrentPage ()); 
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
}

///
/// @brief Test the page navigation using the entry on the toolbar.
///
/// The application has a text entry that the user can use to to a 
/// page just by writing the number. Any non number after the first number
/// character will be discarded.
///
void
MainPterTest::pageNavigationEntry ()
{
    m_View->setOpenFileName ("/tmp/test.pdf");
    m_Document->setNumPages (4);
    m_MainPter->openFileActivated ();
    // Check that sets the correct number of pages and the current page.
    CPPUNIT_ASSERT_EQUAL (4, m_View->getTotalPages ());
    CPPUNIT_ASSERT_EQUAL (1, m_View->getCurrentPage ());
    CPPUNIT_ASSERT (m_View->hasImagePageView ());

    // Now try again using the page entry on toolbar.
    m_View->setGoToPageText ("2 of 4");
    m_MainPter->goToPageActivated ();
    CPPUNIT_ASSERT_EQUAL (2, m_View->getCurrentPage ()); 
    CPPUNIT_ASSERT (m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (m_View->hasImagePageView ());

    m_View->setGoToPageText ("4 of 4");
    m_MainPter->goToPageActivated ();
    CPPUNIT_ASSERT_EQUAL (4, m_View->getCurrentPage ()); 
    CPPUNIT_ASSERT (m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (m_View->hasImagePageView ());    

    m_View->setGoToPageText ("1 of 4");
    m_MainPter->goToPageActivated ();
    CPPUNIT_ASSERT_EQUAL (1, m_View->getCurrentPage ()); 
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (m_View->hasImagePageView ());

    // Invalid values.
    m_View->setGoToPageText ("123123 of 1");
    m_MainPter->goToPageActivated ();
    CPPUNIT_ASSERT_EQUAL (4, m_View->getCurrentPage ()); 
    CPPUNIT_ASSERT (m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (m_View->hasImagePageView ());    

    m_View->setGoToPageText ("0 of 12");
    m_MainPter->goToPageActivated ();
    CPPUNIT_ASSERT_EQUAL (1, m_View->getCurrentPage ()); 
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (m_View->hasImagePageView ());    

    m_MainPter->goToNextPageActivated ();
    m_View->setGoToPageText ("Jejej :-)");
    m_MainPter->goToPageActivated ();
    CPPUNIT_ASSERT_EQUAL (1, m_View->getCurrentPage ()); 
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToFirstPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToLastPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToNextPage ());
    CPPUNIT_ASSERT (m_View->isSensitiveGoToPage ());
    CPPUNIT_ASSERT (!m_View->isSensitiveGoToPreviousPage ());
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
}

///
/// @brief Test the rotation of the page.
///
/// This is a very simple test about page rotation.
///
void
MainPterTest::pageRotate ()
{
    m_View->setOpenFileName ("/tmp/test.pdf");
    m_MainPter->openFileActivated ();
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    CPPUNIT_ASSERT (!m_View->hasImagePageView ());
    CPPUNIT_ASSERT_EQUAL (0, m_Document->getRotation ());

    m_MainPter->rotateRightActivated (); 
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    CPPUNIT_ASSERT (!m_View->hasImagePageView ());
    CPPUNIT_ASSERT_EQUAL (90, m_Document->getRotation ());

    m_MainPter->rotateLeftActivated (); 
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    CPPUNIT_ASSERT (!m_View->hasImagePageView ());
    CPPUNIT_ASSERT_EQUAL (0, m_Document->getRotation ());
}

///
/// @brief Tests page's zoom.
///
/// The DumbDocument has a fixed size of 100x250 pixels and the DumbMainView
/// has a fixed pageView of 75x50. We will make sure that getting to the max
/// ZoomIn insensitives the ZoomIn button. The same for the ZoomOut.
///
/// Then will test Zoom Fit and Zoom Width with two different rotations.
///
void
MainPterTest::pageZoom ()
{
    m_View->setOpenFileName ("/tmp/test.pdf");
    m_MainPter->openFileActivated ();
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    CPPUNIT_ASSERT (!m_View->hasImagePageView ());
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1.0, m_Document->getZoom (), 0.0001);

    m_MainPter->zoomInActivated ();
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    CPPUNIT_ASSERT (!m_View->hasImagePageView ());
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1.2, m_Document->getZoom (), 0.0001);

    m_MainPter->zoomOutActivated ();
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    CPPUNIT_ASSERT (!m_View->hasImagePageView ());
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1.0, m_Document->getZoom (), 0.0001);
   
    CPPUNIT_ASSERT (m_View->isSensitiveZoomIn ());
    CPPUNIT_ASSERT (m_View->isSensitiveZoomOut ());
    // Make sure we go to the last zoom level.
    for (int count = 0 ; count < 20 ; count++)
    {
        m_MainPter->zoomInActivated ();
    }
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomIn ());
    CPPUNIT_ASSERT (m_View->isSensitiveZoomOut ());
    for (int count = 0 ; count < 40 ; count ++)
    {
        m_MainPter->zoomOutActivated ();
    }
    CPPUNIT_ASSERT (m_View->isSensitiveZoomIn ());
    CPPUNIT_ASSERT (!m_View->isSensitiveZoomOut ());

    // OK, now try to zoom width. Since rotation is 0
    // the zoom level should be 50/250 = 0.2
    m_MainPter->zoomFitActivated ();
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    CPPUNIT_ASSERT (!m_View->hasImagePageView ());
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0.2, m_Document->getZoom (), 0.0001);
    // For the width it should be 75/100 = 0.75
    m_MainPter->zoomWidthActivated ();
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    CPPUNIT_ASSERT (!m_View->hasImagePageView ());
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0.75, m_Document->getZoom (), 0.0001);

    // Now rotate and try again.
    m_MainPter->rotateRightActivated (); 
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    CPPUNIT_ASSERT_EQUAL (90, m_Document->getRotation ());
    
    // Since rotation is now 90 the zoom level should be 75/250 = 0.3
    m_MainPter->zoomFitActivated ();
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0.3, m_Document->getZoom (), 0.0001);
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    // For the width it should be 75/250 = 0.3 also.
    m_MainPter->zoomWidthActivated ();
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0.3, m_Document->getZoom (), 0.0001);
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
}

///
/// @brief Test to reload a normal document.
///
/// A "normal" document is a non encrypted document. The document
/// should then be viewed at the same page, scale an rotation.
///
void
MainPterTest::reloadNormal ()
{
    m_View->setOpenFileName ("/tmp/test.pdf");
    m_MainPter->openFileActivated ();
    CPPUNIT_ASSERT (m_View->hasImagePageView ());

    m_MainPter->goToNextPageActivated ();
    m_MainPter->rotateRightActivated (); 
    m_MainPter->zoomWidthActivated ();
    CPPUNIT_ASSERT_EQUAL (2, m_View->getCurrentPage ()); 
    CPPUNIT_ASSERT_EQUAL (90, m_Document->getRotation ()); 
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0.3, m_Document->getZoom (), 0.0001);
    CPPUNIT_ASSERT (m_View->hasImagePageView ());

    // Reload the document.
    m_MainPter->reloadActivated ();
    CPPUNIT_ASSERT_EQUAL (0, 
            g_ascii_strcasecmp ("/tmp/test.pdf", m_View->getTitle ()));
    CPPUNIT_ASSERT_EQUAL (2, m_View->getCurrentPage ()); 
    CPPUNIT_ASSERT_EQUAL (90, m_Document->getRotation ()); 
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0.3, m_Document->getZoom (), 0.0001);
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
}

///
/// @brief Tries to reload an encrypted file.
///
/// Reloading an encrypted file won't ask for the password unless it changed, 
/// but here we assume that don't
///
void
MainPterTest::reloadEncrypted ()
{
    m_View->setOpenFileName ("/tmp/test.pdf");
    m_View->setPassword ("goodpassword");
    m_Document->setTestPassword ("goodpassword");
    m_Document->setOpenError (DocumentErrorEncrypted);
    m_MainPter->openFileActivated ();
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    CPPUNIT_ASSERT_EQUAL (0, 
            g_ascii_strcasecmp ("/tmp/test.pdf", m_View->getTitle ()));
    
    m_MainPter->goToNextPageActivated ();
    m_MainPter->rotateRightActivated (); 
    m_MainPter->zoomWidthActivated ();
    CPPUNIT_ASSERT_EQUAL (2, m_View->getCurrentPage ()); 
    CPPUNIT_ASSERT_EQUAL (90, m_Document->getRotation ()); 
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0.3, m_Document->getZoom (), 0.0001);
    CPPUNIT_ASSERT (m_View->hasImagePageView ());

    // Reload the document.
    m_View->setPassword ("badpassword");
    m_Document->setTestPassword ("goodpassword");
    m_Document->setOpenError (DocumentErrorEncrypted);
    m_MainPter->reloadActivated ();
    CPPUNIT_ASSERT_EQUAL (0, 
            g_ascii_strcasecmp ("/tmp/test.pdf", m_View->getTitle ()));
    CPPUNIT_ASSERT_EQUAL (2, m_View->getCurrentPage ()); 
    CPPUNIT_ASSERT_EQUAL (90, m_Document->getRotation ()); 
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0.3, m_Document->getZoom (), 0.0001);
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    CPPUNIT_ASSERT (!m_View->shownError ());
    CPPUNIT_ASSERT_EQUAL (0, m_View->countTimesShownPasswordPrompt ());
}

///
/// @brief Tries to reload an encrypted file whose password changed.
///
/// Reloading an encrypted file won't ask for the password unless it changed. 
///
void
MainPterTest::reloadChangedPassword ()
{
    m_View->setOpenFileName ("/tmp/test.pdf");
    m_View->setPassword ("goodpassword");
    m_Document->setTestPassword ("goodpassword");
    m_Document->setOpenError (DocumentErrorEncrypted);
    m_MainPter->openFileActivated ();
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    CPPUNIT_ASSERT_EQUAL (0, 
            g_ascii_strcasecmp ("/tmp/test.pdf", m_View->getTitle ()));
    
    m_MainPter->goToNextPageActivated ();
    m_MainPter->rotateRightActivated (); 
    m_MainPter->zoomWidthActivated ();
    CPPUNIT_ASSERT_EQUAL (2, m_View->getCurrentPage ()); 
    CPPUNIT_ASSERT_EQUAL (90, m_Document->getRotation ()); 
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0.3, m_Document->getZoom (), 0.0001);
    CPPUNIT_ASSERT (m_View->hasImagePageView ());

    // Reload the document.
    m_View->setPassword ("newpassword");
    m_Document->setTestPassword ("newpassword");
    CPPUNIT_ASSERT_EQUAL (0,
            g_ascii_strcasecmp("goodpassword", m_Document->getPassword ()));
    m_Document->setOpenError (DocumentErrorEncrypted);
    m_MainPter->reloadActivated ();
    CPPUNIT_ASSERT_EQUAL (0,
            g_ascii_strcasecmp("newpassword", m_Document->getPassword ()));
    CPPUNIT_ASSERT_EQUAL (0, 
            g_ascii_strcasecmp ("/tmp/test.pdf", m_View->getTitle ()));
    CPPUNIT_ASSERT_EQUAL (2, m_View->getCurrentPage ()); 
    CPPUNIT_ASSERT_EQUAL (90, m_Document->getRotation ()); 
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0.3, m_Document->getZoom (), 0.0001);
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
    CPPUNIT_ASSERT (!m_View->shownError ());
    CPPUNIT_ASSERT_EQUAL (1, m_View->countTimesShownPasswordPrompt ());
}

///
/// @brief Checks that the sidebar is shown when a document has outline.
/// 
/// From now, the sidebar is shown only for the outline (no thumbnails),
/// so if a document has outline the sidebar should be shown with it.
///
/// When the presenter needs to show the outline, it just passes the root
/// outline item to the view, and the view must then render it whatever it
/// can (a GtkTreeView in GTK, etc...)
///
void
MainPterTest::showSidebar ()
{
    DocumentOutline *outline = new DocumentOutline ();
    DocumentOutline *child = new DocumentOutline ();
    child->setParent (outline);
    child->setTitle ("Outline 1");
    child->setDestination (2);
    outline->addChild (child);
    m_Document->setPageMode (PageModeOutlines);
    m_Document->setOutline (outline);    
    m_Document->setNumPages (4);
    
    m_View->setOpenFileName ("/tmp/test.pdf");
    m_MainPter->openFileActivated ();
    CPPUNIT_ASSERT (m_View->isShownSidebar ());
    CPPUNIT_ASSERT (outline == m_View->getOutline ());
    CPPUNIT_ASSERT (m_View->hasImagePageView ());

    // When the user does click on a outline item, the view
    // sends to the presenter the outline that has been activated.
    // Then the presenter goes to that page.
    m_MainPter->outlineActivated (child);
    CPPUNIT_ASSERT_EQUAL (2, m_View->getCurrentPage ());
    CPPUNIT_ASSERT (m_View->hasImagePageView ());
}
